#!/usr/bin/fades

"""Maintain a "all the rest" list in Bluesky."""

import datetime
import uuid
import argparse

import atproto  # fades
import infoauth  # fades == 1.5

TOP_LIST_URI = "at://did:plc:nuw6ahzlkka5qexfboivejmv/app.bsky.graph.list/3llgpdo7ima26"
REST_LIST_URI = "at://did:plc:nuw6ahzlkka5qexfboivejmv/app.bsky.graph.list/3ln5z652cgf24"


def refresh_auth(filepath):
    username = input("Username: ")
    password = input("Password: ")

    client = atproto.Client()
    try:
        client.login(username, password)
    except Exception as exc:
        print("======= err!", exc)
        response = getattr(exc, "response", None)
        if response is None:
            raise
        content = response.content
        print("======= content?", content)
        if content.__class__.__name__ != "XrpcError":
            raise
        if content.error != "AuthFactorTokenRequired":
            raise

        # at this point we're ok to 2fa
        print("2FA required!", content.message)
        code = input("Code: ")

        client.login(username, password, auth_factor_token=code)

    # logged in, one way or the other
    token = client.export_session_string()
    print("=======++T", token)
    secrets = {"session_string": token}
    infoauth.dump(secrets, filepath)
    print("Credentials refreshed successfully")


def get_follows(client, my_did):
    """Get all follows."""
    all_follows = []
    cursor = None
    while True:
        response = client.get_follows(my_did, cursor=cursor)
        all_follows.extend(response.follows)
        if response.cursor is None:
            break
        cursor = response.cursor
    return all_follows


def get_list(client, list_uri):
    """Get all items in a list."""
    all_items = []
    query_data = {'list': list_uri}
    while True:
        response = client.app.bsky.graph.get_list(query_data)
        all_items.extend(response.items)
        if response.cursor is None:
            break
        query_data["cursor"] = response.cursor
    return all_items


def main(filepath):
    """Main entry point."""
    auth_info = infoauth.load(filepath)
    session_string = auth_info["session_string"]

    client = atproto.Client()
    client.login(session_string=session_string)
    my_did = client.me.did

    # get follows
    current_follows = get_follows(client, my_did)
    print("Current follows:", len(current_follows))
    current_handles = {profile.handle for profile in current_follows}
    dids_by_handle = {profile.handle: profile.did for profile in current_follows}

    # uncomment this to get lists metadata :)
    # response = client.app.bsky.graph.get_lists({'actor': my_did})
    # for lista in response.lists:
    #     print(f"Lista: {lista.name}")
    #     print(f"  - Propósito: {lista.purpose}")
    #     print(f"  - URI: {lista.uri}")
    #     print(f"  - CID: {lista.cid}")

    # handles in the top list
    items = get_list(client, TOP_LIST_URI)
    top_handles = {item.subject.handle for item in items}
    print("Handles in TOP:", len(top_handles))
    not_top_handles = current_handles - top_handles

    # handles in the rest list
    items = get_list(client, REST_LIST_URI)
    rest_handles = {item.subject.handle for item in items}
    print("Handles in REST:", len(rest_handles))

    # these are in both lists... if it's in 'top', it should not be in 'rest'
    both_handles = rest_handles & top_handles
    print("Handles in both:", len(both_handles))
    for handle in both_handles:
        print("    removing", repr(handle), "--- NEED TO IMPLEMENT")
        # FIXME: implement!!!
        #     return client.com.atproto.repo.delete_record({
        #         "repo": my_did,
        #         "collection": "app.bsky.graph.listitem",
        #         "rkey": lista_item_uri.split('/')[-1]
        #     })

    # these are not in 'top', but neither in 'rest', we should add them to 'rest'
    to_add_handles = not_top_handles - rest_handles
    print("Handles nowhere:", len(to_add_handles))


    # WARNING: WE DO NOT WANT TO DO THIS ANYMORE (but leaving this script as a example usage)
    # for handle in to_add_handles:
    #     print("    adding", repr(handle))
    #     did = dids_by_handle[handle]

    #     now = datetime.datetime.now(datetime.UTC)
    #     record_key = uuid.uuid4().hex + uuid.uuid4().hex
    #     record = dict(
    #         subject=did,
    #         list=REST_LIST_URI,
    #         py_type='app.bsky.graph.listitem',
    #         created_at=now.isoformat(),
    #     )

    #     client.com.atproto.repo.create_record({
    #         "repo": my_did,
    #         "collection": "app.bsky.graph.listitem",
    #         "rkey": record_key,
    #         "record": record,
    #     })


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--auth", action="store_true", help="To refresh the token/session string")
    parser.add_argument("filepath", help="Where to store the token/session_string")
    args = parser.parse_args()

    if args.auth:
        refresh_auth(args.filepath)
    else:
        main(args.filepath)
